%% Empirical kernel mapping of X
% INPUT:
%   X     - Data matrix, columns are samples.
%
%   D  -    order of the EKM
%
%   basis_learning  -   method for finding basis: k-means or FVS
%
%   kernel  - Selection of kernel:
%               'polynomial'
%               'gaussian'
%               'exponential'
%               'sigmoid'
%               'laplace'
%               'multiquad'
%               'invmultiquad'
%  
%   var1    - Kernel parameter
%               for polynomyial kernel:    degree (d = var1)
%               for gaussian kernel:  sigma   (sigma2 = var1)
%   var2    - Kernel parameter
%               for polynomyial kernel:     offset  (c = var2)
%
%OUTPUT:
% 
% X   -  Empirical kernel map of X
% V   - Matrix of basis vectors
%
function [ X_ekm, V_ekm ] = project_data( X, D, basis_learning, kernel, var1, var2)

        X = X/max(max(abs(X))); %scale X
        
        % eliminate small components
        if basis_learning == 1
            Xn=sqrt(sum(X.*X));
            [xmax indmax] = max(Xn);
            Xn = Xn./xmax;
            P=Xn-1/100000; P=sign(P); P=(-P+1)/2;
            Xnp=Xn.*P;
            [xd,Ind]=setdiff(Xn,Xnp);
            XR = X(:,Ind);
            
            % find a basis V in the input pattern space by k-means clustering
            cnum=1e3;
            if (D <= 20)
                epscnum=60.0;
            else
                epscnum=150.0;  % enforce the basis with "small" condition number
            end
            
            while cnum > epscnum
                [SH,Vt]=kmeans(X',D,'distance','sqEuclidean','Replicates',1,'EmptyAction','singleton');
                V = Vt';
                cnum=cond(V)
            end
            clear SH
        elseif basis_learning == 2
            switch(kernel)
                
                case 'polynomial'
                    % polynomial kernel with degree equal to kparam
                    kernfunc = @(X,Y)polykernel(X,Y,var1);
                    
                case 'gaussian'
                    % gaussian kernel with sigma equal to kparam
                    kernfunc = @(X,Y)gausskernel(X,Y,var1);
            end
            rN = D;                 % maximal number of support vectors
            FVSlen = size(X,2);      % number of samples for FVS
            fprintf('***Running FVS ...\n');
            dataFVS = FVSmod( kernfunc, X(:,1:FVSlen)', rN );
            ind     = dataFVS.S;
            V  = X(:,ind);
        end
        
        % use EKM to project input image on a subspace in RKHS
        X_ekm = empirical_kernel_map(X,V,kernel,0,var1,var2);
        V_ekm = empirical_kernel_map(V,V,kernel,0,var1,var2);

end

